--[[ Tardis
     A dimension-editing addon.

     Primary goal: A thing to let you place items relatively.

]]--

local addoninfo, tar = ...

local printf = Library.printf.printf
local printfh = Library.printf.printfh
local printfs = Library.printf.printfs
local sprintf = Library.printf.sprintf
local sprintfh = Library.printf.sprintfh

local cron = Library.LibCron

local player_unit_id = nil
local player_zone = nil

function tar.slashcommand(args)
  printf("No slash command yet.")
end

tar.rarity_color_table = { sellable = { r = .34375, g = .34375, b = 34375 },
	common = { r = .98, g = .98, b = .98 },
	uncommon = { r = 0, g = .797, b = 0 },
	rare = { r = .148, g = .496, b = .977 },
	epic = { r = .676, g = .281, b = .98 },
	relic = { r = 1, g = .5, b = 0 },
	transcendant = { r = .8, g = 0, b = 0 },
	quest = { r = 1, g = 1, b = 0 },
}

function tar.make_clearer(text)
  text:SetEffectGlow({ colorR = 0, colorB = 0, colorG = 0, strength = 2.5, blurX = 6, blurY = 6 })
end

tar.interface_active = false
tar.interface_created = false

tar.FRAME_HEIGHT = 120
tar.FRAME_WIDTH = 270
tar.PADDING = 2

function tar.safe_details(maybe_type)
  status, value = pcall(Inspect.Item.Detail, maybe_type) 
  if status then
    return value
  else
    return { }
  end
end

function tar.show_object(frame, object)
  if not frame then
    return
  end
  if not object or not object.type then
    frame.label:SetText("--")
    frame.label:SetFontColor(0.5, 0.5, 0.5, 1.0)
    frame.icon_border:SetVisible(false)
    for i = 1, #frame.buttons do
      frame.buttons[i]:SetEnabled(false)
      frame.buttons[i]:SetVisible(false)
    end
    for name, field in pairs(frame.fields) do
      field:SetVisible(false)
    end
  else
    local odetails = tar.safe_details(object.type)
    local color = tar.rarity_color_table[odetails.rarity or 'unknown'] or { r = 0.8, b = 0.8, g = 0.8 }
    frame.label:SetText(object.name)
    frame.label:SetFontColor(color.r, color.b, color.g, 1.0)
    frame.item = object
    if object.icon then
      frame.icon:SetTexture("Rift", object.icon)
      frame.icon_border:SetVisible(true)
      frame.icon_border:SetBackgroundColor(color.r, color.b, color.g, 1.0)
    else
      frame.icon_border:SetVisible(false)
    end
    for i = 1, #frame.buttons do
      frame.buttons[i]:SetEnabled(true)
      frame.buttons[i]:SetVisible(true)
    end
    for name, field in pairs(frame.fields) do
      if not frame.anchor then
      end
      if object[name] then
        local value = object[name]
	if not frame.anchor then
	  if tar.anchor_frame.item and tar.anchor_frame.item[name] then
	    value = value - tar.anchor_frame.item[name]
	  end
	end
        field.data:SetText(sprintf("%.2f", value))
      end
      field:SetVisible(true)
    end
  end
end

function tar.clear_anchor()
  tar.show_object(tar.anchor_frame, nil)
end

function tar.set_anchor()
  tar.show_object(tar.anchor_frame, tar.target_frame.item)
  tar.show_object(tar.target_frame, tar.target_frame.item)
end

function tar.create_labeled_field(frame, field, label, xoff, yoff)
  local child = UI.CreateFrame("Frame", "TARDIS frame label", frame)
  child:SetPoint("TOPLEFT", frame, "TOPLEFT", xoff, yoff)
  child.label = UI.CreateFrame("Text", "TARDIS field label", child)
  child.label:SetPoint("TOPRIGHT", child, "TOPLEFT", 40, 0)
  child.label:SetText(label .. ":")
  child.label:SetFontColor(1.0, 0.9, 0.7, 1.0)
  child.label:SetFontSize(child.label:GetFontSize() + 1)
  child.data = UI.CreateFrame("Text", "TARDIS field value", child)
  child.data:SetPoint("TOPLEFT", child, "TOPLEFT", 40 + tar.PADDING, 0)
  child.data:SetText("")
  child.data:SetFontSize(child.data:GetFontSize() + 1)
  tar.make_clearer(child.label)
  tar.make_clearer(child.data)
  frame.fields[field] = child
end

function tar.make_button(frame, corner, xoff, yoff, text, handler)
  local button = UI.CreateFrame("RiftButton", "TARDIS frame button", frame)
  button:SetPoint(corner, frame, corner, xoff, yoff)
  button:SetText("")
  button:SetWidth(60)
  local label = UI.CreateFrame("Text", "TARDIS frame button label", button)
  label:SetPoint("CENTER", button, "CENTER")
  label:SetFontColor(0.9, 0.8, 0.6, 1.0)
  label:SetText(text)
  button:EventAttach(Event.UI.Input.Mouse.Left.Click, handler, "click handler")
  return button
end

function tar.create_object_frame(parent, xoff, yoff, anchor_flag)
  local f
  f = UI.CreateFrame("Frame", "TARDIS object frame", parent)
  f:SetPoint("TOPLEFT", parent, "TOPLEFT", xoff, yoff)
  f:SetWidth(tar.FRAME_WIDTH)
  f:SetHeight(tar.FRAME_HEIGHT)
  if anchor_flag then
    f:SetBackgroundColor(0.3, 0.4, 0.3, 0.5)
  else
    f:SetBackgroundColor(0.4, 0.4, 0.4, 0.5)
  end
  f.label = UI.CreateFrame("Text", "TARDIS object label", f)
  f.label:SetPoint("TOPRIGHT", f, "TOPRIGHT", -tar.PADDING, tar.PADDING)
  f.label:SetFontSize(18)
  tar.make_clearer(f.label)
  f.icon_border = UI.CreateFrame("Frame", "TARDIS object icon border", f)
  f.icon_border:SetPoint("TOPLEFT", f, "TOPLEFT", tar.PADDING, tar.PADDING)
  f.icon = UI.CreateFrame("Texture", "TARDIS object icon", f.icon_border)
  f.icon:SetPoint("TOPLEFT", f.icon_border, "TOPLEFT", tar.PADDING, tar.PADDING)
  f.icon:SetWidth(32)
  f.icon:SetHeight(32)
  f.icon_border:SetPoint("BOTTOMRIGHT", f.icon, "BOTTOMRIGHT", tar.PADDING, tar.PADDING)
  f.anchor = anchor_flag

  f.fields = {}
  tar.create_labeled_field(f, "coordX", "X", -20, 80)
  tar.create_labeled_field(f, "coordY", "Y", 60, 80)
  tar.create_labeled_field(f, "coordZ", "Z", 140, 80)
  tar.create_labeled_field(f, "roll", "R", -20, 100)
  tar.create_labeled_field(f, "pitch", "P", 60, 100)
  tar.create_labeled_field(f, "yaw", "Y", 140, 100)

  f.buttons = {}
  if anchor_flag then
    f.buttons[1] = tar.make_button(f, "TOPRIGHT", -tar.PADDING, tar.PADDING + 20, "CLEAR", tar.clear_anchor)
  else
    f.buttons[1] = tar.make_button(f, "TOPRIGHT", -tar.PADDING, tar.PADDING + 20, "SET", tar.set_anchor)
  end
  tar.show_object(f, nil)
  return f
end

function tar.create_interface()
  tar.context = tar.context or UI.CreateContext("TARDIS")
  tar.frame = UI.CreateFrame("Frame", "TARDIS main frame", tar.context)
  tar.frame:SetPoint("TOPRIGHT", UI.Native.MapMini, "BOTTOMRIGHT", 0, 0)
  tar.frame:SetWidth(tar.FRAME_WIDTH + (tar.PADDING * 2))
  tar.frame:SetHeight(tar.FRAME_HEIGHT * 2 + (tar.PADDING * 3))
  tar.frame:SetBackgroundColor(0.1, 0.1, 0.1, 0.5)
  tar.anchor_frame = tar.create_object_frame(tar.frame, tar.PADDING, tar.PADDING, true)
  tar.target_frame = tar.create_object_frame(tar.frame, tar.PADDING, tar.PADDING * 2 + tar.FRAME_HEIGHT, false)
  tar.interface_created = true
end

function tar.enable_interface()
  if tar.interface_active then
    return
  end
  printf("Okay, enabling EXTRA MAGIC.")
  if not tar.interface_created then
    tar.create_interface()
  end
  tar.interface_active = true
end

function tar.disable_interface()
  if not tar.interface_active then
    return
  end
  printf("Okay, disabling EXTRA MAGIC.")
  tar.interface_active = false
end

function tar.item_changed(eventbits, items)
  local details = Inspect.Dimension.Layout.Detail(items)
  for k, v in pairs(details) do
    if v.selected then
      tar.enable_interface()
      tar.show_object(tar.target_frame, v)
    end
  end
end

function tar.zone_changed(eventbits, units)
  if player_unit_id and units[player_unit_id] then
    local details = Inspect.Unit.Detail('player')
    player_zone = details.zone
    tar.disable_interface()
  end
end

function tar.availability_changed(eventbits, units)
  local details = nil
  if not player_unit_id or not player_zone then
    details = Inspect.Unit.Detail('player')
    if not player_unit_id and details and details.id then
      player_unit_id = details.id
    end
    if not player_zone and details and details.zone then
      player_zone = details.zone
    end
  end
  if units[player_unit_id] then
    if not details then
      details = Inspect.Unit.Detail('player')
    end
    if details.zone ~= player_zone then
      -- Fake zone change event, which we might have missed because
      -- we weren't loaded when it happened.
      tar.zone_changed(nil, { [player_unit_id] = true })
    end
  end
end

Library.LibGetOpt.makeslash("", "Tardis", "tardis", tar.slashcommand)
Command.Event.Attach(Event.Dimension.Layout.Update, tar.item_changed, "detect item updates")
Command.Event.Attach(Event.Unit.Detail.Zone, tar.zone_changed, "detect zone changes")
Command.Event.Attach(Event.Unit.Availability.Full, tar.availability_changed, "detect availability changes")

